############################################################################################
# Chatologica - AddNAVG.pm - add a new node to this software
# All rights reserved (c) 2000; http://www.chatologica.com/
############################################################################################

package DBPUB1::ADMIN::AddNAVG;

use strict;					# use strict pragma
use DBPUB1::STD::Templater;			# load standard routines related to templates
use DBPUB1::STD::Common;                        # load some standard routines
use vars qw(%P %in %out $MODIFIED);		# declare global variables for this package



sub run						# run the main work for this navigation destination
{
    if((!$MODIFIED) or ($MODIFIED != (stat 'DBPUB1/STD/Parameters.pm')[9])){
	local $^W = 0;					# turn warnings OFF for this block
    	do 'DBPUB1/STD/Parameters.pm';                 	# load Parameters package if changed
    	$MODIFIED = (stat 'DBPUB1/STD/Parameters.pm')[9]; 	# Update the LAST MODIFICATION time
    };
    my($input) = @_;
    *P = &DBPUB1::STD::Parameters::get;		# get script parameters
    *in = $input;				# get script input
    &check_input;				# check script input
    &prepare_out;				# makes the %out hash
    print "Content-type: text/html\n\n";	# print response header
    print &eval_template(\%out,"templates/admin/$in{'NAVG'}\.htm");	# show the page
};



sub check_input					# check if CGI Input is correct
{
    if($in{'type'} eq 'ADMIN') {
	$in{'template_dir'} = "admin";
        $in{'access'} = "Administrative";
    } else {
	$in{'template_dir'} = "en";
        $in{'access'} = "Public";
    };
    foreach (keys %in) {
        $in{$_} = &prune_str($in{$_} || '');    # prune white spaces around the string
    };
    if($in{'destination'} !~ /^\w+$/) {
	&DieMsg("The new Navigation Destination Name must be non-empty alphanumeric string!");
    };
    $in{'db_order'} =~ s/\,/ /sg;               # remove commas
    $in{'result_fields'} =~ s/\,/ /sg;
    $in{'db_order'} =~ s/\s\s+/ /sg;		# remove extra spaces
    $in{'result_fields'} =~ s/\s\s+/ /sg;       # remove extra spaces
    if(!$in{'results_template_file'}) {		# set a default search results template file
        $in{'results_template_file'} = "$in{'destination'}-sr.htm";     # a default file name
    };
    if($in{'autogenerate_template'} and !$in{'template_file'}) {
        $in{'template_file'} = "$in{'destination'}.htm";                # a default file name
    };
    if(!$in{'result_fields'}) {                 # no results field names defined for the template
        $in{'result_fields'} = $in{'db_order'}; # by default use the flat db field names if any
    };
    $in{'sql_query1'} =~ s/[\r\n]/ /sg;		# make it in one row
    $in{'sql_query2'} =~ s/[\r\n]/ /sg;
    &security_check($in{'search_criteria'}, $in{'delete_criteria'}, $in{'sort_by_field'},
        $in{'where_clause'}, $in{'order_by_clause'}, $in{'from_clause'}, $in{'sql_query1'},
        $in{'sql_query2'}
    );
};



sub prepare_out					# prepare the output hash %out
{
    %out = ();					# init output - important for mod_perl
    ($out{'script_url'},$out{'nph_script_url'})
        = &get_script_urls(\%P,'1');          	# get the script names
    &make_the_module_file;                      # generate and save new .pm file
    $out{'form'} = "none";                      # still no cgi form
    &make_form_template_file;                   # make a testing cgi form file
    &make_template_file;
    &make_results_template_file;
    $out{'NAVG'} = $in{'destination'};
    if($in{'template_file'}) {
        $out{'output'} = "templates/$in{'template_dir'}/$in{'template_file'}";  # the output html file
    } else {
        $out{'output'} = "<I>none</I>";                                         # no output html file
    };
    $out{'module'} = "DBPUB1/$in{'type'}/$in{'destination'}.pm";                # perl module file
    $out{'access'} = $in{'access'};                     # access type of destination
    $out{'direct_url'} = $out{'script_url'};            # the current script URL, URI or script name
    if($in{'type'} ne "ADMIN") {
        if($P{'script_url'}) {                          # script url is provided as parameter
            $out{'direct_url'} = $P{'script_url'};
        };
        if(!($out{'direct_url'} =~ s{/[^/]+/[^/]+$}{/})){# remove admin/admin_script.cgi from the url
            $out{'direct_url'} = '../';
        };
        $out{'direct_url'} .= 'x.cgi';                  # x.cgi is the default main cgi script name
    };
    if($out{'form'} ne 'none') {
        $out{'test_url'} = $out{'direct_url'};
        $out{'test_url'} .= "?NAVG=ShowTemplate&file=$in{'destination'}-form.htm";
    } else {
        $out{'test_url'} = '';
    };
    $out{'direct_url'} .= "?NAVG=$in{'destination'}";
};



sub make_the_module_file                        # create the perl module implementing the new target
{
    my(%out) = ();				# our local output hash
    $out{'module'} = $in{'destination'};
    $out{'DIR'} = $in{'type'};
    &make_check_input_code(\%out);              # prepare perl code which will check the cgi input
    &make_db_codes(\%out);                      # prepare perl code for db requests subroutine
    &make_print_page_code(\%out);               # prepare perl code that prints response page
    my $new_module_code = &eval_template(\%out,'templates/admin/module-code.txt');      # make module code
    open(F,">DBPUB1/$in{'type'}/$in{'destination'}\.pm") || &DieMsg("Can't write <I>'DBPUB1/$in{'type'}/$in{'destination'}\.pm'</I>: $!");
    print F $new_module_code;                   # saving the new module code as a .pm file
    close F;
    chmod 0666, "DBPUB1/$in{'type'}/$in{'destination'}\.pm";     # make the file writable
}



sub make_check_input_code
{
    my ($out) = @_;
    if($in{'type'} eq "ADMIN") {
        $$out{'default_language'} = '';
        $$out{'is_admin'} = ",'admin'";         # used when have to get script urls
    } else {
        $$out{'default_language'} = 'en';       # english is the default language for public destinations
    };
    foreach (1..30) {                           # we have no more than 30 form fields
        if($in{"form_name$_"}) {                # a form field definition found
            if($in{"check$_"} =~ /non-empty string/) {  # preparing the regular expressions
                $$out{'check_expressions'} .= "         '".$in{"form_name$_"} ."' => '/.+/',\n";
            };
            if($in{"check$_"} =~ /digits only/) {
                $$out{'check_expressions'} .= "         '". $in{"form_name$_"} ."' => '/^\\d+\$/',\n";
            };
            if($in{"check$_"} =~ /url format/) {
                $$out{'check_expressions'} .= "         '". $in{"form_name$_"} ."' => '{^http://.+\$}',\n";
            };
            if($in{"check$_"} =~ /email address/) {
                $$out{'check_expressions'} .= "         '".$in{"form_name$_"} ."' => '{^[^\\\@]+\\\@[^\\\@\.]+\.[^\\\@]+\$}',\n";
            };
            if($in{"label$_"}) {                # make the code for hash of field labels
                $$out{'labels'} .= "            '".$in{"form_name$_"} ."' => '". $in{"label$_"} ."',\n";
            };
            if($in{"default$_"}) {              # make the code for hash of default field values
                $$out{'default_values'} .= "            '".$in{"form_name$_"} ."' => '". $in{"default$_"} ."',\n";
            };
        };
    };
}



sub make_print_page_code                # make perl code which will check the cgi input for the new module
{
    my ($out) = @_;
    my $print_content = '';
    my $print_header = '';
    my $print_footer = '';
    my $send_header = 'print "Content-type: text/html\\n\\n";                                  # print response header';
    if($in{'type'} ne 'ADMIN') {
        $print_header = "print &eval_template(\\%out,\"templates/\$in{'lng'}/header.htm\");     # show the header";
        $print_footer = "print &eval_template(\\%out,\"templates/\$in{'lng'}/footer.htm\"); # show the footer";
	if(!$in{'template_file'}) {
            $print_header = $print_footer = $send_header = '';
            $print_content = "&DieMsg('$in{'response_message'}',\$in{'lng'});";
	} else {
            $print_content = "print &eval_template(\\%out,\"templates/\$in{'lng'}/$in{'template_file'}\");      # show the page";
	};
    } else {
	if(!$in{'template_file'}) {
            $send_header = '';
	    $print_content = "&DieMsg('$in{'response_message'}');";
	} else {
            $print_content = "print &eval_template(\\%out,\"templates/admin/$in{'template_file'}\");    # show the page";
	};
    };

    $$out{'print_page_code'} = <<"endofhtml";      # perl code for the prepare_out subroutine for a module
    $send_header
    $print_header
    $print_content
    $print_footer
endofhtml
}



sub make_db_codes                                       # makes perl code for processing flat databases
{
    my ($out) = @_;                                     # making the default perl codes
    $$out{'page_per_page'} = '';
    $$out{'flat_db_general'} = '';
    $$out{'flat_db_add_row'} = '';
    $$out{'flat_db_delete_rows'} = '';
    $$out{'flat_db_search'} = '';
    $$out{'lock_code'} = '';
    $$out{'unclock_code'} = '';
    $$out{'sql_init'} = '';
    $$out{'sql_query1_code'} = '';
    $$out{'sql_query2_code'} = '';
    $$out{'sql_search_count'} = '';
    $$out{'make_start_end'} = '';
    $$out{'sql_search_code'} = '';
    $$out{'make_db_out'} = '';

    if($in{'db_flat'} && $in{'db_order'} &&
        ($in{'delete_criteria'} || $in{'search_criteria'} || ($in{'add_row'} eq 'Yes'))
        ) {                                             # have to use flat-file database
        $in{'flag_flat_db_used'} = 1;
    };
    if((!defined $in{'flag_flat_db_used'}) &&
        ($in{'sql_query1'} || $in{'sql_query2'} || $in{'from_clause'})
       ) {                                              # have to use sql database
        $in{'flag_sql_db_used'} = 1;
    };
    if($in{'flag_sql_db_used'} && !($in{'db_name'} || $P{'db_name'})) {
	&DieMsg("You requested sql database processing but did not provide the database name. <BR>Please set a default database name through the Admin Panel or return back and set it explicitly in the form!");
    };
    if((defined $in{'flag_flat_db_used'} && $in{'search_criteria'}) ||
       (defined $in{'flag_sql_db_used'} && $in{'from_clause'})
      ) {                                               # we have to produce search results output
         $in{'flag_search_activity'} = 1;
    };

    if(defined $in{'flag_flat_db_used'}) {              # if flat-file db used
        $$out{'flat_db_general'} = <<"endofcode";
    # Flat-file database initialization:
    my (\$db_flat, \@db_structure) = ();
    use DBPUB1::STD::FlatDB;					# loads flat-text database class
    \@db_structure = split(/ /,"$in{'db_order'}"); # order of the fields
    \$db_flat = new DBPUB1::STD::FlatDB('$in{'db_flat'}',\\\@db_structure);
endofcode
        $$out{'flat_db_load'} = <<"endofcode";
    \$db_flat->load;                                   		# load database in memory
endofcode
        if($in{'delete_criteria'}) {
            $$out{'flat_db_delete_rows'} = <<"endofcode";
    my \$delete_criteria = sub {                   # subroutine which returns 1 upon match
	my (\$row) = \@_;
        $in{'delete_criteria'};
    };
    \$db_flat->delete_rows(\$delete_criteria);             	# deleting rows from the database
endofcode
        };
        if($in{'add_row'} eq 'Yes') {
            $$out{'flat_db_add_row'} = <<"endofcode";
    \$db_flat->add_row(\\\%in);                            	# adding a row to the database
endofcode
        };
         if($in{'delete_criteria'} or ($in{'add_row'} eq 'Yes')) {
            $$out{'flat_db_save'} = <<"endofcode";
    \$db_flat->save;                                		# save/update the database file
endofcode
        };
    };
    if(defined $in{'flag_sql_db_used'}) {               # if sql db used	
        $$out{'sql_init'} = <<"endofcode";
    # Prepare a SQL database connection:
    use DBI;						# load Perl's standard database interface
    my (\$sth, \$sql_query, \$query, \$from, \$where, \$order_by, \$host, \$port, \$database) = ();
    \$database = '$in{'db_name'}' || \$P{'db_name'};	# set the database name
    \$host = \$port = '';
    if (\$P{'db_host'}) {				# set the database server host by default
	\$host = ";host=\$P{'db_host'}";
    };
    if (\$P{'db_port'}) {                             	# set the database server port by default
	\$port = ";port=\$P{'db_port'}";
    };
    my \$dbh = DBI->connect("dbi:\$P{'db_driver'}:database=\$database\$host\$port",\$P{'db_user'},\$P{'db_pass'}, { RaiseError => 0, PrintError => 0, AutoCommit => 1 });
    if (not \$dbh){					# database connection failed
	&DieMsg("<b>DataBase connection problem:</b> \$DBI::errstr");
    };

endofcode
	$$out{'sql_quit'} = <<"endofcode";
    \$dbh->disconnect;					# disconnect from database server
endofcode
        $$out{'sub_sql_err'} = <<"endofcode";



sub sql_err
{
    my(\$dbh, \$sth, \$err, \$query, \$fh, \$file) = \@_;
    if(defined \$sth) {\$sth->finish;};			# finish the current database statement
    if((defined \$fh) && (defined \$file)) {
    	&release_lock(\$fh,\$file);
    };				
    \$dbh->disconnect;					# disconnect from database server
    &DieMsg("<B>SQL ERROR:</B> \$err. <P><B>Query:</B> \$query<P>");	# die with an error message
}
endofcode
        if($in{'sql_query1'}) {
	   $$out{'sql_query1_code'} = <<"endofcode";
    # a SQL query:(add or remove records)
    local \$^W = 0; 					# Perl Warnings OFF
    \$sql_query = <<"endofhtml";
$in{'sql_query1'}
endofhtml
    local \$^W = 1;					# Perl Warnings OFF
    \$sth = \$dbh->prepare(\$sql_query) || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$sql_query,\$fh,\$file);
    \$sth->execute() || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$sql_query,\$fh,\$file);
    \$sth->finish();                                 	# finish the database statement
    if(\$DBI::errstr) {                               	# check for error
        &sql_err(\$dbh,\$sth,\$DBI::errstr,\$sql_query,\$fh,\$file);
    };

endofcode
        };
        if($in{'sql_query2'}) {
	    $$out{'sql_query2_code'} = <<"endofcode";
    # a SQL query:(add or remove records)
    local \$^W = 0; 					# Perl Warnings OFF
    \$sql_query = <<"endofhtml";
$in{'sql_query2'}
endofhtml
    local \$^W = 1;					# Perl Warnings OFF
    \$sth = \$dbh->prepare(\$sql_query) || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$sql_query,\$fh,\$file);
    \$sth->execute() || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$sql_query,\$fh,\$file);
    \$sth->finish();                                   # finish the database statement
    if(\$DBI::errstr) {                                        # check for error
        &sql_err(\$dbh,\$sth,\$DBI::errstr,\$sql_query,\$fh,\$file);
    };

endofcode
	};
    };

    if(defined $in{'flag_search_activity'}) {           # if we have search activity for this destination
        $$out{'page_per_page'} = <<"endofcode";
    # Preparing 'page' and 'per_page' values for the search requests:
    \$in{'per_page'} = \$in{'per_page'} || 0;			# to avoid warnings do this initialization:
    \$in{'page'} = \$in{'page'} || 0;
    \$in{'per_page'} =~ s/\D//g;					# fixing per page results number
    \$in{'per_page'} = \$in{'per_page'} || 0;
    \$in{'page'} =~ s/\D//g; \$in{'page'} = \$in{'page'} || 0; 	# fix page number
    if (\$in{'per_page'} < 1) {\$in{'per_page'} = 50;};		# default is 50 results per page
    if (\$in{'page'} < 1) {\$in{'page'} = 1;};			# default page to show is the first
    my \$start_index = \$in{'per_page'}*(\$in{'page'}-1)+1;	# default requested start index
    my (\$found, \$number_of_results, \$number_of_pages) = ();	# variables related to the search results

endofcode
        $$out{'make_db_out'} = <<"endofcode";
    # Generate html componets for the search results table
    use DBPUB1::STD::OutputDB;				# load DataBase output generation tools
    (   \$out{'start'}, \$out{'end'}, \$out{'results'},
                \$out{'number_of_results'}, \$out{'number_of_pages'},
                \$out{'links_to_pages'}, \$out{'page'}
    ) = DBPUB1::STD::OutputDB::results_to_html(		# preparing the database html output
                \$found, \$out{'script_url'}, \\\%in,
                'templates/$in{'template_dir'}/$in{'results_template_file'}', \$number_of_results,
		\$number_of_pages, \$in{'page'}, \$in{'per_page'},
                \$start, \$end );
endofcode
        $$out{'make_start_end'} = <<"endofcode";
    # Prepare the 'start' and 'end' indices:
    \$number_of_pages = int((\$number_of_results + \$in{'per_page'} - 1)/\$in{'per_page'});
    if (\$in{'page'} > \$number_of_pages) {		# impossible page number requested
	\$in{'page'} = \$number_of_pages;			# show the last possible page
    };
    my (\$start, \$end) = (0,0);				# default index numbers
    if (\$in{'page'}) {					# we can show this page
    	\$start = \$in{'per_page'}*(\$in{'page'}-1) + 1;
    };
    if (\$in{'page'} == \$number_of_pages) {		# this is the last result's page
    	\$end = \$number_of_results;			# fix the end index to the last result
    } else {
	\$end = \$start + \$in{'per_page'} - 1;		# set the maximal end index
    };
    \$start_index = \$start -1;

endofcode
        if(defined $in{'flag_flat_db_used'}) {          # if flat-file db used
            $$out{'flat_db_search'} = <<"endofcode";
    # search the flat-file database:
    my \$search_criteria = sub {                   # subroutine which returns 1 upon match
	my (\$row) = \@_;
        $in{'search_criteria'};
    };
    local \$^W = 0;
    (\$found, \$number_of_results) = \$db_flat->search(
        \$search_criteria, undef, \$start_index, \$in{'per_page'}, "$in{'sort_by_field'}", "$in{'sort_descending'}","$in{'sort_numerical'}");
    local \$^W = 1;
endofcode
        };
        if(defined $in{'flag_sql_db_used'}) {           # if sql db used
            $$out{'sql_search_count'} = <<"endofcode";
    # Count the search results number:
    \$from = qq~$in{'from_clause'}~;			# SQL FROM clause
    \$order_by = qq~$in{'order_by_clause'}~;			# SQL ORDER BY clause
    \$where = qq~$in{'where_clause'}~;					# SQL WHERE clause
    \$query = "SELECT COUNT(*) \$from \$where";
    \$sth = \$dbh->prepare(\$query) || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$query);
    \$sth->execute() || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$query);
    \$number_of_results = \$sth->fetchrow();
    \$sth->finish();
    if(\$DBI::errstr) {					# check for error
	&sql_err(\$dbh,\$sth,\$DBI::errstr,\$query);
    };

endofcode
            $$out{'sql_search_code'} = <<"endofcode";
    # Getting the search results:
    \$query = "SELECT * \$from \$where \$order_by LIMIT \$start_index,\$in{'per_page'}";
    \$sth = \$dbh->prepare(\$query) || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$query);
    \$sth->execute() || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$query);
    \$found = \$sth->fetchall_arrayref({});
    \$sth->finish() || &sql_err(\$dbh,\$sth,\$DBI::errstr,\$query);	# finish this statement

endofcode
        };
    };
    if($$out{'sql_query1_code'} || $$out{'sql_query2_code'}              # modification of the database requested
        || $$out{'flat_db_delete_rows'} || $$out{'flat_db_add_row'}      # have to define locks and unlocks
       ) {
	my $lock;							# resource name to lock
	if($$out{'flat_db_delete_rows'} || $$out{'flat_db_add_row'}){	# flat file database locking name
	    $lock = "\"$in{'db_flat'}\"";
   	} else {
	    $lock = "\"$in{'db_name'}\" || '$P{'db_name'}'";		# sql database locking name
	};
        $$out{'lock_code'} = <<"endofcode";
    use DBPUB1::STD::FileLocks;
    my \$resource = $lock;
    (\$fh, \$file) = &get_lock(\$resource);            	# lock the database to avoid corruption
endofcode
        $$out{'unlock_code'} = <<"endofcode";
    &release_lock(\$fh,\$file);                        	# relese the lock on database
endofcode
    };
}



sub make_results_template_file          # make a default template for a search result
{
    my @structure = split(/ /, $in{'result_fields'});   # structure of results
    $out{'sr'} = 'none';
    if(defined $in{'flag_search_activity'}) {
        $out{'sr'} = "templates/$in{'template_dir'}/$in{'results_template_file'}";
    };
    if( (defined $in{'flag_search_activity'}) &&
        ($in{'autogenerate_results_template'} ||
            ! -e "templates/$in{'template_dir'}/$in{'results_template_file'}"
      ) ) {                                             # create a template
        my $result = '';                                # init
        if(@structure) {
            foreach (@structure) {                      # adding $out{'..'} strings for each field
                $result .= "\$out{'$_'} &nbsp;";
            };
        } else {
            $result = 'here is the content of this result';     # there is no fields provided
        };
        my $template = <<"endofhtml";
            <B>\$out{'result_number'}.</B> $result<BR>
endofhtml
        open(F,">templates/$in{'template_dir'}/$in{'results_template_file'}") || &DieMsg("Can't write <I>'templates/$in{'template_dir'}/$in{'results_template_file'}'</I>: $!");
        print F $template;                  # saving the template file
        close F;
        chmod 0666, "templates/$in{'template_dir'}/$in{'results_template_file'}";     # make the file writable
    };
}



sub make_template_file                  # create a default html template for the destination
{
    if($in{'autogenerate_template'} ||                                  # have to generate a file
        ($in{'template_file'} and ! -e "templates/$in{'template_dir'}/$in{'template_file'}")) {
        my(%out) = ();                                                  # our local output hash
        $out{'NAVG'} = $in{'destination'};
        $out{'file'} = "templates/$in{'template_dir'}/$in{'template_file'}"; # the output html file
        $out{'module'} = "DBPUB1/$in{'type'}/$in{'destination'}.pm";    # perl module file
        $out{'access'} = $in{'access'};                                 # access type of destination
        my $template = &eval_template(\%out,'templates/admin/destination-output.htm');
        open(F,">templates/$in{'template_dir'}/$in{'template_file'}") || &DieMsg("Can't write <I>'templates/$in{'template_dir'}/$in{'template_file'}'</I>: $!");
        print F $template;                                              # saving the template file
        close F;
        chmod 0666, "templates/$in{'template_dir'}/$in{'template_file'}";     # make the file writable
    };
}



sub make_form_template_file                        # create a page with cgi form to test the destination
{
    my(%output) = ();                              # our local output hash
    $output{'access'} = $in{'access'};             # access type of destination
    $output{'table'}  = '';                        # to avoid warnings
    my $label_tag = '';
    foreach (1..30) {				# we have no more than 30 form fields
	if($in{"form_name$_"}) {		# a form field definition found
            $label_tag = $in{"label$_"}.":"  if  $in{"label$_"};
            $output{'table'} .= <<"endofhtml";     # making table rows
		<TR>
		   <TD BGCOLOR="#eeeeee"><P ALIGN="Center">$in{"form_name$_"}
		   </TD>
                   <TD><P ALIGN="RIGHT"><B>$label_tag</B>
		   </TD>
		   <TD><INPUT NAME="$in{"form_name$_"}" VALUE="$in{"initial$_"}" SIZE=40>
		   </TD>
		</TR>
endofhtml
        };
    };
    if($output{'table'}) {
        $out{'form'} = "templates/$in{'template_dir'}/$in{'destination'}-form.htm";
    } else {
        return;                         # no html form to create - > exit
    };
    $output{'table'} = <<"endofhtml";      # making the table html code
    <TABLE BORDER="0" CELLPADDING="2" CELLSPACING="1" ALIGN="Center">
      <TR BGCOLOR="#cccccc">
	<TD><P ALIGN=Center>
	  <B><BIG> &nbsp;Field&nbsp;Name&nbsp; </BIG></B></TD>
	<TD><P ALIGN=Center>
	  <B><BIG> &nbsp;Field&nbsp;Label&nbsp; </BIG></B></TD>
	<TD><P ALIGN=Center>
	  <B><BIG> &nbsp;Value&nbsp; </BIG></B></TD>
      </TR>
$output{'table'}
    </TABLE>
endofhtml
    $output{'NAVG'} = &to_HTML($in{'destination'});
    my $template = &eval_template(\%output,'templates/admin/destination-form.htm');
    open(F,">templates/$in{'template_dir'}/$in{'destination'}-form.htm") || &DieMsg("Can't write <I>'templates/$in{'template_dir'}/$in{'destination'}-form.htm'</I>: $!");
    print F $template;			# saving the file
    close F;
    chmod 0666, "templates/$in{'template_dir'}/$in{'destination'}-form.htm";     # make the file writable
}



sub security_check                                              # check if a perl expression is safe
{
    my (@expressions) = @_;
    my $expr;
    foreach $expr (@expressions) {
        if ($expr =~ m/(unlink|open|eval|exec|system|\`|qx|use|require|do|syscall|mkdir|chmod|chown|rename|rmdir|symlink|fork|kill)/) {
            &DieMsg("Perl expression <i><b>'" . &to_HTML($expr) . "'</b></i> did not pass security check. Insert this expression manually in your .pm file.");
        };
    };                                                          # security check passed
}



1; # must return TRUE

