############################################################################################
# Chatologica - Configure.pm - show the parameters configuration page and update parameters
# All rights reserved (c) 2000; http://www.chatologica.com/
############################################################################################

package DBPUB1::ADMIN::Configure;

use strict;					# use strict pragma
use DBPUB1::STD::Templater;			# load standard routines related to templates
use DBPUB1::STD::Common;				# load some standard routines
use vars qw(%P %in %out $MODIFIED);		# declare global variables for this package



sub run						# run the main work for this navigation destination
{
    if((!$MODIFIED) or ($MODIFIED != (stat 'DBPUB1/STD/Parameters.pm')[9])){
	local $^W = 0;					# turn warnings OFF for this block
    	do 'DBPUB1/STD/Parameters.pm';                 	# load Parameters package if changed
    	$MODIFIED = (stat 'DBPUB1/STD/Parameters.pm')[9]; 	# Update the LAST MODIFICATION time
    };
    my($input) = @_;
    *P = &DBPUB1::STD::Parameters::get;		# get script parameters
    *in = $input;				# get script input
    &check_input;				# check script input
    if($in{'update'}) {
    	&set_new_parameters;			# make the requested parameters active
    	my $codes = &make_parameter_codes;	# making perl codes for parameter definitions
    	&save_parameters($codes);		# make new Parameters.pm file
    };
    &prepare_out;				# makes the %out hash
    print "Content-type: text/html\n\n";	# print response header
    print &eval_template(\%out,"templates/admin/$in{'NAVG'}\.htm");	# show the page
};



sub check_input					# check if CGI Input is correct
{
    $ENV{'HTTP_HOST'} = $ENV{'HTTP_HOST'} || '';
    $ENV{'SCRIPT_NAME'} = $ENV{'SCRIPT_NAME'} || '';
};



sub set_new_parameters				# set %P hash with new values
{
    $P{'log_it'} = $in{'log_it'} || 0;			# enable log file
    $P{'htdocs'} = &prune_str($in{'htdocs'} || '');	# path to html docs
    $P{'htdocs'} =~ s{([^/])/$}{$1};			# remove slash at end
    $P{'script_url'} = &prune_str($in{'script_url'} || ''); # path to the script
    $P{'nph_script_url'} = &prune_str($in{'nph_script_url'} || ''); # path to the nph-script
    if($P{'htdocs'} && ! -e $P{'htdocs'}) {		# check directory
	&DieMsg("Directory '$P{'htdocs'}' does not exists. Create it!");
    };
    $P{'db_driver'} = &prune_str($in{'db_driver'} || 'mysql');  # DataBase type
    $in{'custom_db_driver'} = &prune_str($in{'custom_db_driver'} || '');
    if($in{'custom_db_driver'}) {                       # custom database driver provided
        $P{'db_driver'} = $in{'custom_db_driver'};
    };
    $P{'db_host'} = &prune_str($in{'db_host'} || 'localhost');	# DataBase host
    $P{'db_port'} = &prune_str($in{'db_port'} || '');	# DataBase port
    $P{'db_user'} = &prune_str($in{'db_user'} || '');	# DataBase username
    $P{'db_pass'} = &prune_str($in{'db_pass'} || '');	# DataBase password
    $P{'db_name'} = &prune_str($in{'db_name'} || '');   # DataBase password
}



sub prepare_out					# prepare the output hash %out
{
    %out = ();					# init output - important for mod_perl
    ($out{'script_url'},$out{'nph_script_url'})
        = &get_script_urls(\%P,'1');           	# get the script names
    
    $out{'log_it'}{$P{'log_it'}} = " SELECTED";
    $out{'htdocs_root'} = $ENV{'DOCUMENT_ROOT'} || '/home/httpd/html/';
    $out{'htdocs_root'} =~ s{\\}{/}g; $out{'htdocs_root'} =~ s{([^/])$}{$1/}g;
    $out{'htdocs'} = $P{'htdocs'};
    $out{'the_script_url'} = $P{'script_url'};
    $out{'the_nph_script_url'} = $P{'nph_script_url'};
    $out{'script_path'} = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}";
    $out{'script_path'} =~ s{:\d{1,6}/}{/};		# remove port number
    $out{'script_path'} =~ s{/[^/]+/[^/]+$}{/};		# switch 2 directory levels up
    $out{'db_driver'}{$P{'db_driver'}} = " SELECTED";
    $out{'custom_db_driver'} = $P{'db_driver'};
    $out{'db_host'} = $P{'db_host'};
    $out{'db_port'} = $P{'db_port'};
    $out{'db_user'} = $P{'db_user'};
    $out{'db_pass'} = $P{'db_pass'};
    $out{'db_name'} = $P{'db_name'};
    $out{'this_server'} = $ENV{'HTTP_HOST'} || 'localhost';
    $out{'this_server'} =~ s/:.*//;                      # remove port number
    local $^W = 0;
    local $SIG{__DIE__} = undef;
    $out{'db_drivers'} = eval("use DBI; return join(' ', DBI->available_drivers);") || '';
};



sub make_parameter_codes			# generate Perl codes which define each
{						# parameter value. Return it as hash
    my(%codes) = ();
    $codes{'log_it'} = "'$P{'log_it'}'";
    $codes{'htdocs'} = "'$P{'htdocs'}'";
    $codes{'script_url'} = "'$P{'script_url'}'";
    $codes{'nph_script_url'} = "'$P{'nph_script_url'}'";
    $codes{'db_driver'} = "'$P{'db_driver'}'";
    $codes{'db_host'} = "'$P{'db_host'}'";
    $codes{'db_port'} = "'$P{'db_port'}'";
    $codes{'db_user'} = "'$P{'db_user'}'";
    $codes{'db_pass'} = "'$P{'db_pass'}'";
    $codes{'db_name'} = "'$P{'db_name'}'";
    return \%codes;
}



sub save_parameters			# update parameters.pl
{
    my($codes) = @_;
    my $new_parameters = <<"end_of_parameters";
#+START OF PARAMETERS UPDATABLE THROUGH ADMIN PANEL - DO NOT ALTER THIS LINE!

    # Enable log file logs/log.txt?
    # example: \$P{'log_it'} = '0'; 	# disabled
    \$P{'log_it'} = $$codes{'log_it'};

    # Path to html documents directory dedicated to this application:
    # example: \$P{'htdocs'} = '/home/httpd/html/my_docs';
    \$P{'htdocs'} = $$codes{'htdocs'};

    # URL to the main script. Leave it empty to use the real one.
    # example: \$P{'script_url'} = 'http://chatologica.com/cgi-bin/script/x.cgi';
    \$P{'script_url'} = $$codes{'script_url'};

    # URL to the nph-script. Leave it empty to use the real one.
    # example: \$P{'script_url'} = 'http://chatologica.com/cgi-bin/script/nph-x.cgi';
    \$P{'nph_script_url'} = $$codes{'nph_script_url'};

    # DataBase Server Driver
    # example: \$P{'db_driver'} = 'mysql';
    \$P{'db_driver'} = $$codes{'db_driver'};

    # DataBase Server host.
    # example: \$P{'db_host'} = 'localhost';
    \$P{'db_host'} = $$codes{'db_host'};

    # DataBase Server Port.
    # example: \$P{'db_port'} = '3345';
    \$P{'db_port'} = $$codes{'db_port'};

    # DataBase username.
    # example: \$P{'db_user'} = 'root';
    \$P{'db_user'} = $$codes{'db_user'};

    # DataBase password.
    # example: \$P{'db_pass'} = 'fdfghj';
    \$P{'db_pass'} = $$codes{'db_pass'};

    # DataBase name.
    # example: \$P{'db_name'} = 'chatologica';
    \$P{'db_name'} = $$codes{'db_name'};

end_of_parameters
    open(F,'<DBPUB1/STD/Parameters.pm') || &DieMsg("Can't read DBPUB1/STD/Parameters.pm file. $!<BR>Check if this file exists!");
    my @content = <F>;			# read the old parameters file
    close F;
    my $content = join('',@content);
    if ($content !~ s/^(.*)(\#\+START OF PARAMETERS.*)(\#\+END OF PARAMETERS.*)$/$1$new_parameters$3/s) {
	&DieMsg("You have removed the important stringw '#+START OF PARAMETERS' and '#+END OF PARAMETERS' from your Parameters.pm file and now it is not possible to determine automaticaly where exactly the updated parameters code should be included.".
	"Pease either restore an original Parameters.pm from an original archive or put these strings in Parameters.pm around the place where the Admin Panle generated parameters should be inserted.");
    } else {
   	open(F,'>DBPUB1/STD/Parameters.pm') || &DieMsg("Can't write to DBPUB1/STD/Parameters.pm file. $!<BR>Check if this file has world-write permission!");
	print F "$content";		# save the new content
	close(F);
        chmod 0666, 'DBPUB1/STD/Parameters.pm';         # make the file writable
    };
};



1; # must return TRUE

