#########################################################################################
# Chatologica - CGI_Parser.pm - reading and parsing the CGI input into %hash
# All rights reserved (c) 2000; http://www.chatologica.com/
#########################################################################################

package DBPUB1::STD::CGI_Parser;                # read and parse the CGI input data

use strict;					# use strict pragma



sub GetInput					# returns a reference to the input hash
{                                               # keys are the parameters, vals are their values
    my(%in) = ();				# input data hash here
    my $cgi_max_data = 250000;                  # maximum bytes to accept via POST
    my $type = $ENV{'CONTENT_TYPE'} || '';	# Getting several useful ENV variables
    my $length  = $ENV{'CONTENT_LENGTH'} || 0;
    my $method = $ENV{'REQUEST_METHOD'} || '';
    if($length > $cgi_max_data) {
	die("CGI_Parser: http request to receive too much data: $length bytes\n");
    };
    if(!(!$method || $method =~ /GET|POST/i || $type eq 'application/x-www-form-urlencoded')) {
	die("CGI_Parser: unsupported http request method: $method");
    };
    if($type =~ m{multipart/form-data}i) {
	die("CGI_Parser: unsupported http content type: $type");
    };

    my $input = $ENV{'QUERY_STRING'} || '';		# read input data
    if($method =~ /POST/i) {  				# if POST method
        read(STDIN, $input, $length);                   # read from STDIN
    };

    my @params = split(/[&;]/, $input);		# split the pairs
    push(@params, @ARGV); 			# add command-line parameters
    my($i, $key, $val) = ();			# init temp variables
    foreach $i (0..$#params) {
	$params[$i] =~ s/\+/ /g;     				# Convert plus to space
	($key, $val) = split(/=/, $params[$i], 2);  		# Split into key and value
	if(defined $key) {
	    $val = $val || '';					# empty value is default
	    $key =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;	# Convert %XX from hex
	    $val =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;	# numbers to alphanumeric
	    $in{$key} .= "\0" if (defined($in{$key})); 		# \0 is the multiple separator
	    $in{$key} .= $val;	   				# Associate key and value
	};
    };
    return \%in;	# return a reference to this hash (the input data)
};



1; # must return TRUE

