#########################################################################################
# Chatologica - Common.pm - some frequently used procedures
# All rights reserved (c) 2000; http://www.chatologica.com/
#########################################################################################

package DBPUB1::STD::Common;

use strict;				# use strict pragma



sub to_HTML				# convert a plain text to HTML, encoding the
{ 					# &, ", < and > symbols
    my($str) = @_;
    $str =~ s/\&/\&amp;/sg;
    $str =~ s/</\&lt;/sg;
    $str =~ s/>/\&gt;/sg;
    $str =~ s/\"/&quot;/sg;
    return $str				# return encoded string
};



sub to_text				# turn a html code to plain text
{
    my($str) = @_;			# the html code
    $str =~ s/<p>|<br>|<dt>|<dd>/ /sg;	# translate line terminators to spaces
    $str =~ s/<.+?>//sg;		# remove HTML tags
    $str =~ s/[\n\r]/ /sg;		# remove new lines
    $str =~ s/&nbsp;/ /sg;		# translate white spaces
    $str =~ s/&amp;/&/sg;		# translate & signs
    $str =~ s/&quot;/\"/sg;		# translate " signs
    $str =~ s/&lt;/</sg;                # translate < signs
    $str =~ s/&gt;/>/sg;                # translate > signs
    $str =~ s/\s+/ /sg;                 # shrink white spaces
    $str =~ s/^\s+(\S)/$1/;             # remove beginning white spaces
    return $str;			# return plain text
};



sub prune_str			# remove spaces at begining and at the end of a string
{
    my($str) = @_;
    $str =~ s/^\s+(\S)/$1/s;	# remove left spaces
    $str =~ s/(\S)\s+$/$1/s;	# remove right spaces
    $str =~ s/^[\s\t]+$//s;	# empty it if it is only white space
    return $str;
}



sub URLencode				# URL encoding of illegal characters
{
    my($s) = @_;			# string to encode
    my($str, $c, $c1, $c2, $char) = ();
    $str = '';
    my @h = ('A'..'F');			# hex numbers
    while ($s =~ m/(.)/sg) {
	$char = $1;
	if($char !~ /([\w\. ])/) { 	# encode only non-word symbols
	    $c = ord $char;
	    $c1 = $c % 16; 		# making first hex digit
	    if($c1 > 9) {
		$c1 = $h[$c1 - 10];
    	    };
            $c2 = int ($c / 16);      	# making the second hex digit
    	    if($c2 > 9) {
		$c2 = $h[$c2 - 10];
    	    };
	    $char = "%$c2$c1";		# this is the hex code of the symbol
	};
	$str .= $char;
    };
    $str =~ s/ /+/g;			# encode white spaces with + signs
    return $str;        		# return URL-encoded string
};



sub import {			# this function is called automatically by use Module; statement
    my $pkg = shift;
    my $callpkg = caller(0);	# import these symbols into the caller package:
    no strict 'refs';
    foreach $_ (qw(to_HTML to_text prune_str URLencode)) {
    	*{"${callpkg}::$_"} = \&{"${pkg}::$_"};
    };
}



1; # must return TRUE

