#########################################################################################
# Chatologica - Cookies.pm - handle cookies
# All rights reserved (c) 2000; http://www.chatologica.com/
#########################################################################################

package DBPUB1::STD::Cookies;

use strict;				# use strict pragma
use DBPUB1::STD::Common;


sub getCookies				# return hash with all cookies received
{
    my(%Cookies,$name,$value) = ();
    foreach (split(/; /,$ENV{'HTTP_COOKIE'})) {
	($name,$value) = split(/=/);
	$Cookies{&URLdecode($name)} = &URLdecode($value);
    };
    return %Cookies;
};



sub setCookie				# print a Set-Cookie: header line
{
    my($name, $value, $expiration, $path, $domain, $secure) = @_;
    $secure=' secure' if($secure);
    if($expiration) {			# you provide the expiration date either as number
	if($expiration !~ /GMT/) {	# of seconds since 1970 or as GMT standard date format
	    $expiration = &cookie_date($expiration);
	};
	$expiration = " expires=$expiration;";
    };
    $path = " path=$path;" if($path);
    $domain = " domain=$domain;" if($domain);
    $name = &URLencode($name);
    $value = &URLencode($value);
    print "Set-Cookie: $name=$value;$expiration$path$domain$secure\n";
}



sub compressCookies			# compress a hash into a string to send as cookie
{
    my(%cookies) = @_;
    my(@compressed) = ();
    foreach (keys %cookies) {
	push @compressed, &URLencode($_) ."=". &URLencode($cookies{$_});
    };
    return join('&',@compressed);
}



sub decompressCookie				# return hash of decompressed cookies
{
    my($cookie) = @_;
    my(%Cookies) = ();
    my @params = split(/[&]/, $cookie);		# split the pairs
    my($i, $key, $val) = ();			# init temp variables
    foreach $i (0..$#params) {	
	$params[$i] =~ s/\+/ /g;     				# Convert plus to space
	($key, $val) = split(/=/, $params[$i], 2);  		# Split into key and value      	
	$key =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;	# Convert %XX from hex
	$val =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;	# numbers to alphanumeric
	$Cookies{$key} = $val;	   				# Associate key and value
    };
    return %Cookies;
};



sub URLdecode				# turn any %hex encoding into real chars
{
    my($str) = @_;
    $str =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
    return $str;
}

   

sub cookie_date				# make the time in cookie standard format
{					# $time - number of seconds since 1970
    my($time)= @_;
    my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst)= gmtime($time); # in GMT
    my(@months)= ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
    my(@weekday)=("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday");
    $sec = "0$sec" if ($sec < 10);
    $min = "0$min" if ($min < 10);
    $hour = "0$hour" if ($hour < 10);
    $mday = "0$mday" if ($mday < 10);
    $year += 1900;  
    return "$weekday[$wday], $mday-$months[$mon]-$year $hour\:$min\:$sec GMT";
};



sub import {			# this function is called automatically by use Module; statement
    my $pkg = shift;
    my $callpkg = caller(0);	# import these symbols into the caller package:
    no strict 'refs';
    foreach $_ (qw(setCookie getCookies compressCookies decompressCookie)) {	
    	*{"${callpkg}::$_"} = \&{"${pkg}::$_"};
    };
}



1; # must return TRUE

