############################################################################################
# Chatologica - OutputDB.pm - prepare an html output for DataBase search results
# All rights reserved (c) 2000; http://www.chatologica.com/
############################################################################################

package DBPUB1::STD::OutputDB;

use strict;
use DBPUB1::STD::Common;			# load frequently used procedures
use DBPUB1::STD::Templater;			# load standard routines related to templates



sub results_to_html                          	# prepare components for the resulted html page
{                                               # as a standard search engine output
    my ($found, $script_url, $in, $template_file, $number_of_results, $number_of_pages, $page,
	$per_page, $start, $end) = @_;
    my ($results, $links_to_pages) = ();
    if ($page) {				# we have to show this page
    	if (open(F,"<$template_file")) {	# read results layout template
    	    my @txt = <F>;
    	    close F;
    	    my $template = join('',@txt);			# make it as a string
	    my (%result, $num) = ();
    	    foreach $num (0..($end-$start)) {			# all search results for the page
		if (defined $$found[$num]) {
	    	    %result = %{$$found[$num]};			# copy the search result
    	            $result{'result_number'} = $start + $num;	# current result number
    	            $results .= &eval_template(\%result,'',$template);	# make html
		};
    	    };
	};
    } else {
	$results = '<I>no results found</I>';			# init results output code
    };
    $links_to_pages = &make_links_to_pages($page, $number_of_pages, $script_url, $in);
    return ($start, $end, $results, $number_of_results, $number_of_pages, $links_to_pages, $page);
};



sub make_links_to_pages				# generate html code for links to all
{						# search result pages
    my($page, $number_of_pages, $script_url, $in) = @_;
    my($code) = ();				# the html code to generate
    if($page) {					# there are search results/pages
	my($p, @codes, $next_page_code, $prev_page_code, $url_string) = ();
	$prev_page_code = '';
	$next_page_code = '';
	foreach (keys %$in) {			# start making the URL string
	    if($_ ne 'page') {			# exclude the page parameter - will set later
		$url_string .= &URLencode($_) . "=" . &URLencode($$in{$_}) . '&';
	    };
	};
	$url_string = &to_HTML("$script_url?$url_string");	# add the script path
    	foreach $p (1..$number_of_pages) {	# pass through all of the page numbers
	    if($p != $page) {			# link it
		push @codes,"<a href=\"$url_string" . "page=$p\">$p</a>";
	    } else {
		push @codes,"<B>$p</B>";	# do not link the page number
	    };
	};
	if($page > 1) {				# have to link to previous page
	    $p = $page - 1;
	    $prev_page_code = "<a href=\"$url_string"."page=$p\">[&lt;&lt;]</a>";
	};
	if($page < $number_of_pages) {		# have to link to next page
	    $p = $page + 1;
	    $next_page_code = "<a href=\"$url_string"."page=$p\">[&gt;&gt;]</a>\n";
	};
	$code = join("&nbsp; \n", ($prev_page_code, @codes, $next_page_code));	# put spaces
    } else {
	$code = 'none';				# no results - no pages
    };
    return $code;				# return the html code of the page links
};



1; # must return TRUE

